
/*++ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* CALL:
*   SKin BinSockAddr(ap,ip)
*   	char *ap;
*   	SKin *ip;
*
* DESCRIPTION:
*
* This routine attempts to decode an IP address.  White space is stripped off
* and  we  try  to  decode as many numeric fields as we can.  If this doesn't
* work, we try gethostbyname(), and hope that it recognizes the string.   The
* result  is  stored  in  ip's  IPaddr  field, and the address type is set to
* Internet.
*
* The return value is the 32-bit IP address, or zero for failure.
*
* Several formats are accepted, using either symbolic names or dotted-decimal
* notation. We also try to recognize a final field that is a port number, and
* fill in the sin_port field.
*
* AUTHOR:
*   John Chambers.
++*/
#include "V.h"
#include "sys_in.h"
#include "sys_netdb.h"
global char du4fmt[] = "%u.%u.%u.%u";
global char du5fmt[] = "%u.%u.%u.%u.%u";

IP32 binsockaddr(ap,ip)
	char *ap;
	SKin *ip;
{	int   c, dots;
	U32   r=0;
	Uint  n;
	char* dp[5];
	char* cp;
	U32   u1, u2, u3, u4, u5;
	HOST* hp;
	Fenter("binsockaddr");
	ChkNull(ap,"address pointer");
	ChkNull(ip,"sockaddr_in*");
	n = Strlen(ap);
	V6 "%d-byte address %s",n,Dsps(ap,n) D;
	while ((c = *ap) && isspace(c)) ++ap;
	V6 "%d-byte address \"%s\" [after isspace]",n,ap D;
	n = Strtrim(ap);
	V6 "%d-byte address \"%s\" [after Strtrim]",n,ap D;
	ChkNull(n,"address string");
	for (dots=0, cp=ap; c = *cp; cp++) {	/* Count the dots */
		if (c == '.') {
			dp[Min(dots,4)] = cp;
			++dots;
		}
	}
	V6 "There are %d dots in %s",dots,Dsps(ap,-1) D;
	switch(dots) {
	case 4:
		if (sscanf(ap,du5fmt,&u1,&u2,&u3,&u4,&u5) == 5) {
			V6 "Got 6 numbers: %d.%d.%d.%d.%d.",u1,u2,u3,u4,u5 D;
			cp = (CP)&(ip->sin_addr.s_addr);
			*cp++ = u1;
			*cp++ = u2;
			*cp++ = u3;
			*cp   = u4;
			ip->sin_port = u5;
			r = IPhost(ip);
			setIPfmly(ip,AF_INET);
			V6 "Arg: %08X:%04X=%s.",IPhost(ip),IPport(ip),SymSockAddr(ip) D;
			Done;
		}
		V6 "du5fmt didn't match %s",Dsps(N(ap),-1) D;
		*dp[4] = 0;	/* Trim away the final numeric field */
	case 3:
		if (sscanf(ap,du4fmt,&u1,&u2,&u3,&u4) == 4) {
			V6 "Got 4 numbers: %d.%d.%d.%d.",u1,u2,u3,u4 D;
			cp = (CP)&(ip->sin_addr.s_addr);
			*cp++ = u1;
			*cp++ = u2;
			*cp++ = u3;
			*cp   = u4;
			r = ip->sin_addr.s_addr;
			ip->sin_port = 0;	/* Unknown port; should get filled in */
			setIPfmly(ip,AF_INET);
			V6 "Arg: %08X:%04X=%s.",IPhost(ip),IPport(ip),SymSockAddr(ip) D;
			Done;
		}
		V6 "du4fmt didn't match %s",Dsps(N(ap),-1) D;
		break;
	default:
		if (hp = Gethostbyname(ap)) {
			V6 "Got host entry for %s",Dsps(ap,-1) D;
			Bcopy(hp->h_addr, &r, hp->h_length);
			setIPfmly(ip,hp->h_addrtype);
		/*	IPhost(ip) = r; */
			ip->sin_addr.s_addr = htonl(r);
			V6 "Address %08X=%s for %s",r,SymSockAddr(ip),Dsps(ap,-1) D;
			Done;
		}
	}
	V2 "Unknown address format in %s",Dsps(ap,-1) D;
fail:
	V4 "Failed for %s",Dsps(ap,-1) D;
	r = 0;
done:
	V5 "Return %08X=%s.",r,ipdot(&r) D;
	Fexit;
	return r;
}
