#include "V_s_lc.h"

#define SLEEPLIMIT 30

/*** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* CALL:
*   d_wwrite(f,b,n)
*   	int	f;		-- IN: File to receive data
*   	BP	b;		-- IN: Buffer full of data
*   	int	n;		-- IN: Number of bytes to write
*
* DESCRIPTION:
*   Waiting Write.  This routine is a  wrapper  around  write()  which
*   waits  until  the  kernel  has accepted all the data, or returns a
*   fatal error.  What we do is write as much as we can, and if it  is
*   less  than  n,  we try again for the remnant.  If we get an ENOSPC
*   ("No space left on device") error, we put out a Vlvl 2  message,
*   wait  a  short  time,  and  try  again.   This  routine is a prime
*   candidate for tweaking for particular systems.
*
* RETURNS:
*   The  same  as  write(),  the number of bytes written if successful
*   (which should be the same as n), or -1 and an errno  code.   Which
*   errno codes are possible varies from system to system.
*
* AUTHOR: John Chambers.
***/
FCT d_wwrite(f,b,n)
	int	f;		/* IN: File to receive data */
	BP	b;		/* IN: Buffer full of data */
	int	n;		/* IN: Number of bytes to write */
{	int	e=errno,	/* Remember and possibly restore error code */
		l=n,		/* Number of bytes left to write */
		r=0,		/* Return value, number of bytes written */
		s=1,		/* Sleep time */
		w=0;		/* Intermediate write result */
	BP	p=b;		/* Next byte to write */

	Fenter("wwrite");
:5	V5 "wwrite(%d,%X,%d)",f,b,n D;
	while (l > 0) {		/* While anything left to write */
		w = Write(f,p,l);
		if (w > 0) {
			l -= w;	/* Bytes left */
			r += w;	/* Bytes written, return value */
			p += w;	/* Next byte to write */
		} elsif (w <= 0) {
			e = errno;
:5			V5s "Write(%d,%X,%d)=%d [Err %d=%s=%s]",f,b,n,w,Errinfo D;
			Switch(e) {
			case ENOSPC:
			case EWOULDBLOCK:
:2				V2 "Write to %s failing [Err %d=%s=%s]",FileName(f),Errinfo D;
				Sleep(s);
				if (++s > SLEEPLIMIT) s = SLEEPLIMIT;
				break;
			default:
				Fail;
			}
		}
	}
done:
	Freturn(r);
fail:
:2	errno = e;
:2	V2 "Write to %s failed [Err %d=%s=%s]",FileName(f),Errinfo D;
	errno = e;	/* Paranoia */
	Freturn(-1);
}
